package biga.utils;
import nme.Vector;
import nme.geom.Point;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class GeomUtils
{
	static private var ip:Point = new Point();
	/**
	 * returns the angle between p0 and p1
	 * @param	p0
	 * @param	p1
	 * @return
	 */
	static public function angle( p0:Point, p1:Point ):Float
	{
		return Math.atan2( p1.y - p0.y, p1.x - p0.x );
	}
	/**
	 * returns the difference between two angles
	 * @param	a0 first angle
	 * @param	a1 second angle
	 * @return
	 */
	static public function angleDifference( a0:Float, a1:Float ):Float
	{
		var difference:Float = a1 - a0;
		while (difference < -Math.PI ) difference += Math.PI * 2;
		while (difference > Math.PI ) difference -= Math.PI * 2;
		return difference;
	}
	/**
	 * returns the slope between p0 and p1
	 * @param	p0
	 * @param	p1
	 * @return
	 */
	static public function slope( p0:Point, p1:Point ):Float
	{
		return ( p1.y - p0.y ) / ( p1.x - p0.x );
	}
	/**
	 * returns the distance between p0 & p1
	 * @param	p0
	 * @param	p1
	 * @return
	 */
	static public function distance( p0:Point, p1:Point ):Float
	{
		return Math.sqrt( squareDistance( p0.x, p0.y, p1.x, p1.y ) );
	}
	/**
	 * returns the square distance between 2 sets of x / y values
	 * @param	x0
	 * @param	y0
	 * @param	x1
	 * @param	y1
	 * @return
	 */
	static public function squareDistance( x0:Float, y0:Float, x1:Float, y1:Float ):Float
	{
		return ( x0 - x1 ) * ( x0 - x1 ) + ( y0 - y1 ) * ( y0 - y1 );
	}
	/**
	 * returns the value with a floating point the length of step
	 * @param	value 
	 * @param	step : number of figures after the point. 10 - .1, 100-0.01 etc
	 * @return
	 */
	static public function toFixed( value:Float, step:Float = 10 ):Float
	{
		return Math.floor( ( value * step ) ) / step ;
	}
	/**
	 * reutrn the value number of times the step fits entirely into the value without returning a float
	 * @param	value 
	 * @param	step 
	 * @return
	 */
	static public function snap( value:Float, step:Float ):Float
	{
		return Math.floor( value / step ) * step;
	}
	/**
	 * clamps a value between 2 values min / max
	 * @param	value
	 * @param	min
	 * @param	max
	 * @return
	 */
	static public function clamp( value:Float, min:Float, max:Float ):Float
	{
		return value < min ? min : ( value > max ? max : value );
	}
	/**
	 * linear interpolation of T between 2 values A and B
	 * @param	t
	 * @param	a
	 * @param	b
	 * @return
	 */
	static public function lerp( t:Float, a:Float, b:Float ):Float
	{
		return a + ( b - a ) * t;
	}
	/**
	 * normalizes the value between A and B
	 * @param	value
	 * @param	a
	 * @param	b
	 * @return
	 */
	static public function normalize( value:Float, a:Float, b:Float ):Float
	{
		return ( value - a ) / ( b - a );
	}
	/**
	 * maps a value between 2 sets of values
	 * @param	value
	 * @param	min1
	 * @param	max1
	 * @param	min2
	 * @param	max2
	 * @return
	 */
	static public function map(value:Float, min1:Float, max1:Float, min2:Float, max2:Float):Float
	{
		return lerp( normalize( value, min1, max1 ), min2, max2 );
	}
	
	/**
	 * returns a point at T along the array of points
	 * @param	t ratio 
	 * @param	points a list of points
	 * @param	p (optional) returned point object
	 * @return
	 */
	private function getPointAt( t:Float, points:Vector<Point>, p:Point = null, loop:Bool = false ):Point 
	{
		var length:Int = points.length;
		if ( !loop ) length--;
		
		var i:Int = Math.floor( length * t );
		var delta:Float = 1 / ( length );
		t = ( t - ( i * delta ) ) / delta;
		
		if ( p == null )
		{
			p =  new Point( 	lerp( t, points[ i ].x, points[ ( i + 1 ) % points.length ].x ),
								lerp( t, points[ i ].y, points[ ( i + 1 ) % points.length ].y ) );
		}
		else
		{
			p.x = lerp( t, points[ i ].x, points[ ( i + 1 ) % points.length ].x );
			p.y = lerp( t, points[ i ].y, points[ ( i + 1 ) % points.length ].y );
		}
		return p;
	}
	
	/**************************************************************
			VECTOR UTILS
	**************************************************************/
	static public function determinant( p:Point, a:Point, b:Point ):Float
	{
		return ( ( a.x - b.x ) * ( p.y - b.y ) ) - ( ( p.x - b.x ) * ( a.y - b.y ) );
	}
	static public function isLeft( p:Point, a:Point, b:Point ):Bool
	{
		return determinant( p, a, b ) >= 0;
	}
	static public function isRight( p:Point, a:Point, b:Point ):Bool
	{
		return determinant( p, a, b ) <= 0;
	}
	static public function normalizeVector( p:Point ):Point
	{
		return new Point( p.x / p.length, p.y / p.length );
	}
	static public function dotProduct( u:Point, v:Point ):Float
	{
		return ( u.x * v.x + u.y * v.y );
	}
	static public function crossProduct( u:Point, v:Point ):Float
	{
		return ( u.x * v.y - u.y * v.x );
	}
	static public function center( p0:Point, p1:Point ):Point
	{
		return new Point( p0.x + ( p1.x - p0.x ) / 2, p0.y + ( p1.y - p0.y ) / 2 );
	}
	static public function normal( p0:Point, p1:Point ):Point
	{
		return new Point( -( p1.y - p0.y ), ( p1.x - p0.x ) );
	}
	static public function leftNormal( p0:Point, p1:Point ):Point
	{
		return new Point( p0.x + ( p1.y - p0.y ), p0.y - ( p1.x - p0.x ) );
	}
	static public function rightNormal( p0:Point, p1:Point ):Point
	{
		return new Point( p0.x - ( p1.y - p0.y ), p0.y + ( p1.x - p0.x ) );
	}
	static public function reflect( p:Point, a:Point, b:Point ):Point
	{
		var n:Point = normal( a, b );
		n.normalize(1);
		var dot2:Float = 2 * GeomUtils.dotProduct( p, n );
		return new Point( 	p.x - dot2 * n.x,	p.y - dot2 * n.y );
	}
	
	
	
	/**
	 * 
	 * @param	p0 vector origin
	 * @param	p1 vector direction
	 * @param	l0 line start
	 * @param	l1 line end
	 * @return
	 */
	static public function bounceVector( p0:Point, p1:Point, l0:Point, l1:Point, PasSegment:Bool = false, LasSegment:Bool = true ):Array<Point>
	{
		var pp:Point = GeomUtils.lineIntersectLine( p0, p1, l0, l1, PasSegment, LasSegment );
		if ( pp != null )
		{
			var n:Point = normal( l0, l1 ).add( pp );
			var m:Point = reflect( pp.subtract( p1 ), pp, n ).add( pp );
			return [ pp, m, m.subtract( pp ) ];
		}
		return null;
	}
	
	/**
	 * returns the distance between point P and line A - B
	 * @param	p
	 * @param	a
	 * @param	b
	 * @return
	 */
	static public function pointLineDistance( p:Point, a:Point, b:Point ):Float
	{	
		
		return distance( p, project( p, a, b ) );
		
	}
	
	/**
	 * perpendicular projection of P on the line or segment A-B
	 * @param	p	point to project
	 * @param	a	start of the line or segment
	 * @param	b	end of the line or segment
	 * @param	asSegment	specifies if A B should be a segment or a line
	 * @return the perpendicular pojection of P on A-B
	 */
	
	static public function project( p:Point, a:Point, b:Point, asSegment:Bool = true ):Point
	{
		
		var dx:Float = b.x - a.x;
		var dy:Float = b.y - a.y;
		
		if ( asSegment && dx == 0 && dy == 0 )
		{
			return a;
		}
		
		var t:Float = ( ( p.x - a.x ) * dx + ( p.y - a.y ) * dy) / ( dx * dx + dy * dy );
		
		if ( asSegment && t < 0) 
		{
			return a;
		} 
		else if ( asSegment && t > 1) 
		{
			return b;
		}
		else 
		{
			return new Point( a.x + t * dx, a.y + t * dy );
		}
		return null;
	}
	/**
	 * retrieves and returns the closest point in a point's list from a given point P
	 * @param	p		the point to test
	 * @param	list	the points list
	 * @return the closest point from P in the list 
	 */
	static public function getClosestPointInList( p:Point, list:Vector<Point> ):Point
	{
		if ( list.length <= 0 ) return null;
		if ( list.length == 1 ) return list[ 0 ];
		var dist:Float;
		var min:Float = Math.POSITIVE_INFINITY;
		var closest:Point = null;
		var item:Point;
		for ( item in list) 
		{
			if ( item == p )continue;
			if ( item.equals( p ) ) return item;
			dist = squareDistance( p.x, p.y, item.x, item.y );
			if ( dist < min ) 
			{
				min = dist;
				closest = item;
			}
		}
		return closest;
	}
	/*
	---------------------------------------------------------------------------
	http://keith-hair.net/blog/
	Returns the closest Point on Line "AB" to Point "p":
	p			-Point, the Point to varrain to the line.
	A			-Point, beginning Point of Line.
	B			-Point, ending Point of Line.
	as_seg	-Boolean, limits return Point between endpoints of A and B.
	----------------------------------------------------------------------------
	*/
	static public function varrain( p:Point, a:Point, b:Point ):Point
	{
		var dx:Float = b.x - a.x;
		var dy:Float = b.y - a.y;
		if (dx == 0 && dy == 0) 
		{
			return a;
		}
		else
		{
			var t:Float = ( (p.x - a.x) * dx + (p.y - a.y) * dy) / (dx * dx + dy * dy);
			t = Math.min( Math.max( 0, t ), 1 );
			return new Point(	a.x + t * dx, a.y + t * dy	);
		}
		return null;
	}
	/*
	---------------------------------------------------------------
	http://keith-hair.net/blog/
	Checks for intersection of Segment if as_seg is true.
	Checks for intersection of Line if as_seg is false.
	Return intersection of Segment AB and Segment EF as a Point
	Return null if there is no intersection
	---------------------------------------------------------------
	*/
	static public function lineIntersectLine (	A : Point, B : Point,
												E : Point, F : Point,
												ABasSeg:Bool = true, EFasSeg:Bool = true ) : Point
	{
		var a1:Float, a2:Float, b1:Float, b2:Float, c1:Float, c2:Float;
		a1= B.y-A.y;
		b1= A.x-B.x;
		a2= F.y-E.y;
		b2= E.x-F.x;
		var denom:Float=a1*b2 - a2*b1;
		if (denom == 0)
		{
			return null;
		}
		c1= B.x*A.y - A.x*B.y;
		c2 = F.x * E.y - E.x * F.y;
		ip = new Point();
		ip.x=(b1*c2 - b2*c1)/denom;
		ip.y=(a2*c1 - a1*c2)/denom;
		if ( A.x == B.x )
			ip.x = A.x;
		else if ( E.x == F.x )
			ip.x = E.x;
		if ( A.y == B.y )
			ip.y = A.y;
		else if ( E.y == F.y )
			ip.y = E.y;
		if ( ABasSeg )
		{
			if ( ( A.x < B.x ) ? ip.x < A.x || ip.x > B.x : ip.x > A.x || ip.x < B.x )
				return null;
			if ( ( A.y < B.y ) ? ip.y < A.y || ip.y > B.y : ip.y > A.y || ip.y < B.y )
				return null;
		}
		if ( EFasSeg )
		{
			if ( ( E.x < F.x ) ? ip.x < E.x || ip.x > F.x : ip.x > E.x || ip.x < F.x )
				return null;
			if ( ( E.y < F.y ) ? ip.y < E.y || ip.y > F.y : ip.y > E.y || ip.y < F.y )
				return null;
		}
		return ip;
	}
	
	/**
	 * remove duplicate points from the points Array
	 */
	static public function removeDuplicates( points:Vector<Point> ):Vector<Point> 
	{
		for ( i in 0...points.length )
		{
			var j:Int = ( i + 1 );
			while( j < Std.int( points.length ) )
			{
				if ( points[ i ].equals( points[ j ] ) )
				{
					points.splice( j, 1 );
					if ( j > ( i + 1 ) )
					{
						j--;
						continue;
					}
				}
				j++;
			}
		}
		
		return points;
	}
}