package h3d.models.sparse;
import biga.color.Color;
import biga.color.HSV;
import biga.utils.GeomUtils;
import h3d.material.WireFrame;
import h3d.models.Model;
import nme.display.BitmapData;
import nme.Vector;
using nme.Vector;
/**
 * @author Nicolas Barradeau 
 */

class SparseModel extends Model
{
	
	private var velocities:Vector<Float>;
	private var _colors:Vector<Int>;
	private var hsv:HSV;
	
	private var _repulsion:Float = 1;
	private var _attraction:Float = .05;
	private var _damping:Float = .15;
	
	public function new() 
	{
		super();
		hsv = new HSV( 0, 100, 100 );
		material = new WireFrame( 0, 0xFF0000 );
	}
	
	public function initFromData( edges:Array<Int> ):Void
	{
		indices = Preprocessor.processData( edges );
		reset();
	}
	
	public function initFromBitmap( bd:BitmapData ):Void
	{
		indices = Preprocessor.processBitmap( bd );
		reset();
	}
	
	private function reset():Void
	{
		
		var vCount:Int = indices[ indices.length - 1 ];
		indices.pop();
		
		colors = new Vector<Int>();
		vertices = new Vector<Float>();
		velocities = new Vector<Float>();
		for( i in 0...vCount )
		{
			
			vertices.push( ( Math.random() - .5 ) );
			vertices.push( ( Math.random() - .5 ) );
			vertices.push( ( Math.random() - .5 ) );
			
			colors.push( 0 );
			
			velocities.push( 0 );
			velocities.push( 0 );
			velocities.push( 0 );
			
		}
		
	}
	
	
	public function update():Void
	{
		if ( indices == null || vertices == null ) return;
		
		var px:Float = 0;
		var py:Float = 0;
		var pz:Float = 0;
		
		var ox:Float = 0;
		var oy:Float = 0;
		var oz:Float = 0;
		
		var dx:Float = 0;
		var dy:Float = 0;
		var dz:Float = 0;
		var dist:Float = 0;
		
		var coulomb:Float = 0;
		
		var i:Int = 0;
		var j:Int = 0;
		var vCount:Int = Std.int( vertices.length );
		
		//compute repulsion
		while( i < vCount ) // loop through vertices
		{
			
			px = vertices[ i ];
			py = vertices[ i + 1 ];
			pz = vertices[ i + 2 ];
			
			j = 0;
			while( j < vCount )
			{
				
				if ( i == j )
				{
					j += 3;
					continue;
				}
				
				ox = vertices[ j ];
				oy = vertices[ j + 1 ];
				oz = vertices[ j + 2 ];
			
				//distance between p and other
				dx = ( px - ox );
				dy = ( py - oy );
				dz = ( pz - oz );
				
				dist = ( dx * dx + dy * dy + dz * dz );
				
				//avoid division by 0
				if (dist == 0) dist = 0.001;
				
				//apply repulsion
				coulomb = repulsion / dist;
				
				velocities[ i ] += dx * coulomb;
				velocities[ i + 1 ] += dy * coulomb;
				velocities[ i + 2 ] += dz * coulomb;
				
				j += 3;
				
			}
			
			i += 3;
			
		}
		
		
		//compute attraction
		i = 0;
		var id0:Int, id1:Int;
		while ( i < Std.int( indices.length ) )
		{
			
			//p
			id0 = indices[ i ] * 3;
			px = vertices[ id0 ];
			py = vertices[ id0 + 1 ];
			pz = vertices[ id0 + 2 ];
			
			//other
			id1 = indices[ i + 1 ] * 3;
			ox = vertices[ id1 ];
			oy = vertices[ id1 + 1 ];
			oz = vertices[ id1 + 2 ];
			
			//edge length
			dx = ( ox - px );
			dy = ( oy - py );
			dz = ( oz - pz );
				
			//push p towrds other 
			velocities[ id0 ] += attraction * dx;
			velocities[ id0 + 1 ] += attraction * dy;
			velocities[ id0 + 2 ] += attraction * dz;
			
			//pull other towards p
			velocities[ id1 ] -= attraction * dx;
			velocities[ id1 + 1 ] -= attraction * dy;
			velocities[ id1 + 2 ] -= attraction * dz;
			
			//stores the color
			hsv.hue = GeomUtils.map( repulsion / Math.sqrt( dx * dx + dy * dy + dz * dz ), 1, 0, 0, 360 );
			colors[ Std.int( i / 2 ) ] = Color.hsv_rgb( hsv );
			
			i += 2;
			
		}
		
		//assign new positions & collect bounding rect
		for( i in 0...vCount )
		{
			vertices[ i ] = ( vertices[ i ] + velocities[ i ] ) * damping; 
		}
		
		
	}
	
	private function get_colors():Vector<Int> 
	{
		return _colors;
	}
	
	private function set_colors(value:Vector<Int>):Vector<Int> 
	{
		return _colors = value;
	}
	
	public var colors(get_colors, set_colors):Vector<Int>;
	
	private function get_repulsion():Float 
	{
		return _repulsion;
	}
	
	private function set_repulsion(value:Float):Float 
	{
		return _repulsion = value;
	}
	
	public var repulsion(get_repulsion, set_repulsion):Float;
	
	private function get_attraction():Float 
	{
		return _attraction;
	}
	
	private function set_attraction(value:Float):Float 
	{
		return _attraction = value;
	}
	
	public var attraction(get_attraction, set_attraction):Float;
	
	private function get_damping():Float 
	{
		return _damping;
	}
	
	private function set_damping(value:Float):Float 
	{
		return _damping = value;
	}
	
	public var damping(get_damping, set_damping):Float;
	
	
}