package triga.spline.shapes;
import nme.Vector;
using nme.Vector;
import away3d.core.base.SubGeometry;
import away3d.core.base.SubMesh;
import away3d.entities.Mesh;
import away3d.materials.MaterialBase;
import nme.geom.Matrix3D;
import nme.geom.Vector3D;
import triga.utils.VectorUtils;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class PTFSpline extends SubGeometry
{
	private var tangents:Vector<Vector3D>;
	private var _path:Vector<Vector3D> = new Vector<Vector3D>();
	private var _radius:Float;
	private var _sides:Int;
	private var _capHoles:Bool;
	private var _section:Section;
	// reference axis to calc orientation aka magic varant, tweak at your own risk
	private var WORLD_UP_AXIS:Vector3D = Vector3D.Z_AXIS;
	private var material:MaterialBase;
	private var _mesh:Mesh;
	private var PI_div_2:Float = Math.PI * .5;
	static public var RADIANS_TO_DEGREES:Float = 180 / Math.PI;
	static public var ZERO:Vector3D = new Vector3D();
	/**
	 * creates a 3D Spline object.
	 * @param	path a series of Vector3D objects describing the path
	 * @param	radius specifies the radius of the spline mesh
	 * @param	sides specifies the number of sides of the spline mesh
	 * @param	capHoles specifies if faces should be created at the ends of the spline mesh
	 * @param	material [optional]	specifies a material to assign to the renderable spline mesh 
	 */
	public function new( path:Vector<Vector3D>, radius:Float = 2, sides:Int = 3, capHoles:Bool = true, material:MaterialBase = null ) 
	{
		this.material = material;
		_radius = radius;
		_sides = sides;
		_capHoles = capHoles;
		_section = Section.createRegularSection( _radius, _sides );
		_section.loop = false;
		this.path = path;//calls compute();
	}
	/**
	 * magic
	 */
	private function compute():Void
	{
		var vs:Vector<Float> = new Vector<Float>();
		var uvs:Vector<Float> = new Vector<Float>();
		var indices:Vector<Int> = new Vector<Int>();
		//orientation
		var v:Vector3D, T1:Vector3D, T2:Vector3D, A:Vector3D, angle:Float = 0;
		var sectionVectors:Vector<Vector3D>;
		//indices
		var step:Int = 0, back:Int, i:Int, j:Int, i0:Int, i1:Int, i2:Int, i3:Int;
		//builds the first frame 
		getFirstFrame();
		sectionVectors = _section.vectors3D;
		for( v in sectionVectors )
		{
			vs.push( v.x, v.y, v.z );
		}
		step = sides;
		//stores the original orientation
		var m:Matrix3D = _section.transform.clone();
		for ( i = 1; i < path.length )
		{
			T1 = tangents[ i-1 ];
			T2 = tangents[ i ];
			A = T1.crossProduct( T2 );
			if ( A.equals( ZERO ) ) continue;
			angle = Math.acos( T1.dotProduct( T2 ) ) * 180 / Math.PI ;
			m.appendRotation( angle, A );
			_section.transform = m;
			_section.position = path[ i ];
			//push transformed vertices
			sectionVectors = _section.vectors3D;
			for( v in sectionVectors )
			{
				vs.push( v.x, v.y, v.z );
			}
			//creating indices
			if ( step > 0 )
			{
				back = step - sides;
				for ( j in 0... sides )
				{
					i0 = back + j;
					i1 = back + ( j + 1 ) % sides;
					i2 = step + j;
					i3 = step + ( j + 1 ) % sides;
					indices.push( i1, i0, i2 );
					indices.push( i1, i2, i3 );
				}
			}
			step += sides;
		}
		uvs = VectorUtils.expand2D( Vector<Float>([0, 1]), Vector<Float>([0, 1]), path.length-1, sides );
		//closing the ends if needed
		if ( capHoles )
		{
			var len:Int = vs.length / 3 - sides;
			for ( i in 0... _section.indices.length / 2; i+=3 )
			{
				indices.push( _section.indices[ i + 1 ], _section.indices[ i ], _section.indices[ i + 2 ] );
				indices.push( len + _section.indices[ i ], len + _section.indices[ i + 1 ], len + _section.indices[ i + 2 ] );
			}
		}
		indices = indices.concat( indices.copy().reverse() );
		//updates geometry
		updateVertexData( vs );
		updateIndexData( indices );
		autoDeriveVertexNormals = true;
		updateUVData( uvs );
	}
	private function getFirstFrame():Void 
	{	
		//finds the orientation of the first section
		var v:Vector3D = path[ 0 ];
		var n:Vector3D = path[ 1 ];
		var dir:Vector3D = n.subtract( v );
		dir.normalize();
		var angle:Float = Math.acos( WORLD_UP_AXIS.dotProduct( dir ) );
		var axis:Vector3D = WORLD_UP_AXIS.crossProduct(dir);
		axis.normalize();
		_section.transform.identity();
		_section.rotate( axis, angle * RADIANS_TO_DEGREES );
		_section.position = path[ 0 ];
	}
	/**
	 * recomputes the vertices positions only
	 */
	public function update():Void
	{
		var vs:Vector<Float> = new Vector<Float>();
		var v:Vector3D, T1:Vector3D, T2:Vector3D, A:Vector3D, angle:Float = 0;
		var sectionVectors:Vector<Vector3D>;
		getFirstFrame();
		sectionVectors = _section.vectors3D;
		for( v in sectionVectors )	vs.push( v.x, v.y, v.z );
		var m:Matrix3D = _section.transform.clone();
		for ( var i:Int = 1; i < path.length )
		{
			T1 = tangents[ i-1 ];
			T2 = tangents[ i ];
			A = T1.crossProduct( T2 );
			if ( A.equals( ZERO ) ) continue;
			angle = Math.acos( T1.dotProduct( T2 ) ) * 180 / Math.PI ;
			m.appendRotation( angle, A );
			_section.transform = m;
			_section.position = path[ i ];
			sectionVectors = _section.vectors3D;
			for( v in sectionVectors )vs.push( v.x, v.y, v.z );
		}
		updateVertexData( vs );
	}
	/*******************************************
	properties
	*******************************************/
	public var path( get_path, set_path ):Vector;
	public function get_path():Vector<Vector3D> 
	{
		return _path;
	}
	public function set_path(value:Vector<Vector3D>):Vector 
	{
		//remove duplicate locations along the path
		_path = value;
		var i:Int = _path.length - 1;
		while ( i-->1 )
		{
			if ( _path[ i - 1 ].nearEquals( _path[i], .1 ) )
			{
				_path.splice( i, 1 );
				return path;
	}
		}
		//precomputing the tangents
		tangents = new Vector<Vector3D>();
		tangents.push( _path[ 1 ].subtract( _path[ 0 ] ));
		for ( i = 1; i < _path.length - 1) tangents.push( _path[ i ].subtract( _path[ i - 1 ] ) );
		tangents.push( _path[ _path.length - 1 ].subtract( _path[ _path.length - 2 ] ));
		for( var v:Vector3D in tangents ) v.normalize();
		compute();
	}
	public var radius( get_radius, set_radius ):Float;
	public function get_radius():Float 
	{
		return _radius;
	}
	public function set_radius(value:Float):Float 
	{
		_radius = value;
		_section = Section.createRegularSection( radius, sides );
		compute();
		return radius;
	}
	public var sides( get_sides, set_sides ):Int;
	public function get_sides():Int 
	{
		return _sides;
	}
	public function set_sides(value:Int):Int 
	{
		_sides = Math.max( 3, value );
		_section = Section.createRegularSection( radius, sides );
		compute();
		return sides;
	}
	public var capHoles( get_capHoles, set_capHoles ):Bool;
	public function get_capHoles():Bool 
	{
		return _capHoles;
	}
	public function set_capHoles(value:Bool):Bool 
	{
		_capHoles = value;
		compute();
		return capHoles;
	}
	/*************************************************
		utils
	*************************************************/
	public var mesh( get_mesh, null ):Mesh;
	public function get_mesh():Mesh
	{
		if ( _mesh == null )
		{
			_mesh = new Mesh();
			return addToMesh( _mesh );
		}
		return _mesh;
	}
	public function addToMesh( mesh:Mesh ):Mesh
	{
		mesh.geometry.addSubGeometry( this );
		var subMesh:SubMesh = mesh.getSubMeshForSubGeometry( this );
		subMesh.material = material;
		return mesh;
	}
}