package triga.spline.shapes;
import nme.Vector;
using nme.Vector;
import away3d.core.base.SubGeometry;
import away3d.entities.Mesh;
import away3d.materials.MaterialBase;
import nme.geom.Matrix;
import nme.geom.Point;
import nme.geom.Vector3D;
import triga.spline.utils.Triangulator2D;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Section extends Mesh
{
	private var original:Vector<Vector3D>;
	private var vectors:Vector<Vector3D>;
	private var transforms:Vector<Vector3D>;
	private var _points:Vector<Point>;
	private var _vertices:Vector<Float>;
	private var _indices:Vector<Int>;
	private var _uvs:Vector<Float>;
	private var _loop:Bool = true;
	public function new( points:Vector<Point>, material:MaterialBase = null ) 
	{
		super( material );
		_points = points;
		compute();
	}
	/**
	 * computes the Section's shape
	 */
	private function compute():Void
	{
		var subGeometry:SubGeometry = new SubGeometry();
		original = new Vector<Vector3D>();
		vectors = new Vector<Vector3D>();
		transforms = new Vector<Vector3D>();
		vertices = new Vector<Float>();
		uvs = new Vector<Float>();
		var min:Point = new Point(Math.POSITIVE_INFINITY, Math.POSITIVE_INFINITY);
		var max:Point = new Point(Math.NEGATIVE_INFINITY, Math.NEGATIVE_INFINITY);
		//computes the indices
		indices = Triangulator2D.compute( points );
		//might be null..
		if( indices  == null ) indices = Vector<Int>([]);
		indices = indices.concat( indices.copy().reverse() );
		//make vertices loop for uv mapping
		if ( loop && !points[0].equals( points[points.length - 1] ) ) points.push( points[0] );
		//store vertices
		for (i in 0 ... points.length) 
		{
			var p:Point = points[ i ];
			var v:Vector3D = new Vector3D( p.x, -p.y, 0 );
			original.push( v );
			vectors.push( v.clone() );
			transforms.push( v );
			vertices.push( Math.floor( p.x + .5 ), Math.floor( -p.y + .5 ), 0 );
			if ( p.x < min.x ) min.x = p.x;
			if ( p.y < min.y ) min.y = p.y;
			if ( p.x > max.x ) max.x = p.x;
			if ( p.y > max.y ) max.y = p.y;
		}
		//uvs
		var dx:Float = 1 / ( max.x - min.x );
		var dy:Float = 1 / ( max.y - min.y );
		for( i in 0... points.length) 
		{
			p = points[ i ];
			uvs.push( ( p.x - min.x) * dx, ( p.y - min.y ) * dy );
		}
		//creates geometry if needed
		if ( geometry.subGeometries.length == 0 )
		{
			geometry.addSubGeometry( new SubGeometry() );
		}
		geometry.subGeometries[ 0 ].updateVertexData( vertices );
		geometry.subGeometries[ 0 ].updateVertexNormalData( vertices.copy() );
		geometry.subGeometries[ 0 ].autoDeriveVertexNormals = true;
		geometry.subGeometries[ 0 ].updateUVData( uvs );
		geometry.subGeometries[ 0 ].updateIndexData( indices );
	}
	/**
	 * creates a section of arbitrary shape after the given set of x/y doublets
	 * @param	values a series of X/Y coordinates
	 * @param	material [ optional ] the material to assign to the Section object
	 * @return a Section Object
	 */
	static public function createFromValues( values:Vector<Float>, material:MaterialBase = null ):Section
	{
		var points:Vector<Point> = new Vector<Point>();
		for ( i in 0 ... values.length; i+=2 )
		{
			points.push( new Point( values[ i ], values[ i + 1 ] ) );
		}
		return new Section( points, material );
	}
	/**
	 * short hand to create a regular polygon
	 * @param	radius	the radius of the section
	 * @param	sides	the number of sides
	 * @param	material [ optional ] a material to assign to the section
	 * @return a Section object
	 */
	static public function createRegularSection( radius:Float = 100, sides:Int = 12, material:MaterialBase = null ):Section
	{
		var points:Vector<Point> = new Vector<Point>();
		sides = Math.max( 3, sides );
		for ( i in 0 ... sides )
		{
			var a:Float = i * ( Math.PI * 2 ) / sides;
			points.push( new Point( Math.cos( a ) * radius, Math.sin( a ) * radius ) );
		}
		return new Section( points, material );
	}
	/**
	 * applies a local 2D transfomration
	 * @param	matrix transfomation matrix to apply to the section
	 */
	public function transform2D( matrix:Matrix ):Void
	{
		for ( i in 0 ... points.length )
		{
			var p:Point = matrix.transformPoint( points[ i ] );
			vectors[ i ].x = p.x;
			vectors[ i ].y = -p.y;
		}
	}
	/******************************************
		properties
	******************************************/
	/**
	 * @return the series of Vector3D transformed by the object's transform matrix
	 */
	public var vectors3D( get_vectors3D, null ):Vector;
	public function get_vectors3D():Vector<Vector3D>
	{
		for ( i in 0 ... vectors.length )
		{
			var v:Vector3D = vectors[ i ];
			transforms[ i ] = transform.transformVector( v );
		}
		return transforms;
	}
	public var points( get_points, set_points ):Vector;
	public function get_points():Vector<Point> 
	{
		return _points;
	}
	public function set_points(value:Vector<Point>):Vector 
	{
		_points = value;
		return points;
	}
	public var vertices( get_vertices, set_vertices ):Vector;
	public function get_vertices():Vector<Float> 
	{
		return _vertices;
	}
	public function set_vertices(value:Vector<Float>):Vector 
	{
		_vertices = value;
		return vertices;
	}
	public var indices( get_indices, set_indices ):Vector;
	public function get_indices():Vector<Int> 
	{
		return _indices;
	}
	public function set_indices(value:Vector<Int>):Vector 
	{
		_indices = value;
		return indices;
	}
	public var uvs( get_uvs, set_uvs ):Vector;
	public function get_uvs():Vector<Float> 
	{
		return _uvs;
	}
	public function set_uvs(value:Vector<Float>):Vector 
	{
		_uvs = value;
		return uvs;
	}
	public var loop( get_loop, set_loop ):Bool;
	public function get_loop():Bool 
	{
		return _loop;
	}
	public function set_loop(value:Bool):Bool 
	{
		_loop = value;
		if ( points[0].equals( points[points.length - 1] ) ) points.pop();
		compute();
		return loop;
	}
}