package triga.spline.shapes;
import nme.Vector;
using nme.Vector;
import away3d.core.base.SubGeometry;
import away3d.core.base.SubMesh;
import away3d.entities.Mesh;
import away3d.materials.MaterialBase;
import nme.geom.Vector3D;
import triga.utils.VectorUtils;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Spline extends SubGeometry
{
	private var _path:Vector<Vector3D>;
	private var _radius:Float;
	private var _sides:Int;
	private var _capHoles:Bool;
	private var _section:Section;
	// reference axis to calc orientation aka magic varant, tweak at your own risk
	private var FIXED_UP_AXIS:Vector3D = Vector3D.Z_AXIS;
	private var material:MaterialBase;
	private var _mesh:Mesh;
	private var PI_div_2:Float = Math.PI * .5;
	static public var RADIANS_TO_DEGREES:Float = 180 / Math.PI;
	/**
	 * creates a 3D Spline object.
	 * @param	path a series of Vector3D objects describing the path
	 * @param	radius specifies the radius of the spline mesh
	 * @param	sides specifies the number of sides of the spline mesh
	 * @param	capHoles specifies if faces should be created at the ends of the spline mesh
	 * @param	material [optional]	specifies a material to assign to the renderable spline mesh 
	 */
	public function new( path:Vector<Vector3D>, radius:Float = 2, sides:Int = 3, capHoles:Bool = true, material:MaterialBase = null ) 
	{
		this.material = material;
		_path = path;
		_radius = radius;
		_sides = sides;
		_capHoles = capHoles;
		_section = Section.createRegularSection( _radius, _sides );
		_section.loop = false;
		compute();
	}
	/**
	 * magic
	 */
	private function compute():Void
	{
		var vs:Vector<Float> = new Vector<Float>();
		var uvs:Vector<Float> = new Vector<Float>();
		var indices:Vector<Int> = new Vector<Int>();
		//orientation
		var v:Vector3D, n:Vector3D, dir:Vector3D, angle:Float = 0;
		var sectionVectors:Vector<Vector3D>;
		//indices
		var step:Int = 0, back:Int, i:Int, i0:Int, i1:Int, i2:Int, i3:Int;
		for ( i in 0... path.length )
		{
			v = path[ i ];
			n = path[ ( i + 1 ) >= path.length ? i : ( i+1 ) ];
			//find the orientation of the section at this location
			//http://www.experts-exchange.com/Programming/Game/3D_Prog./Q_24783637.html
			if ( !v.equals( n ) )
			{
				dir = v.subtract( n );
				dir.normalize();
				angle = Math.acos( FIXED_UP_AXIS.dotProduct( dir ) ) * 180 / Math.PI;
				if ( angle > -PI_div_2 || angle < PI_div_2 ) 
				{
					dir = n.subtract( v );
					dir.normalize();
					angle = Math.acos( FIXED_UP_AXIS.dotProduct( dir ) );
				}
				dir = FIXED_UP_AXIS.crossProduct(dir);
				dir.normalize();
				_section.transform.identity();
				_section.rotate( dir, angle * RADIANS_TO_DEGREES );
			}
			//positions the section at the current location
			_section.position = v;
			//pushing vertices
			sectionVectors = _section.vectors3D;
			for( v in sectionVectors )
			{
				vs.push( v.x, v.y, v.z );
			}
			//creating indices
			if ( step > 0 )
			{
				back = step - sides;
				for ( j in 0 ... sides )
				{
					i0 = back + j;
					i1 = back + ( j + 1 ) % sides;
					i2 = step + j;
					i3 = step + ( j + 1 ) % sides;
					indices.push( i1, i0, i2 );
					indices.push( i1, i2, i3 );
				}
			}
			step += sides;
		}
		uvs = VectorUtils.expand2D( Vector<Float>([0, 1]), Vector<Float>([0, 1]), path.length, sides );
		//closing the ends if needed
		if ( capHoles )
		{
			var len:Int = vs.length / 3 - sides;
			for ( i in 0... _section.indices.length / 2; i+=3 )
			{
				indices.push( _section.indices[ i + 1 ], _section.indices[ i ], _section.indices[ i + 2 ] );
				indices.push( len + _section.indices[ i ], len + _section.indices[ i + 1 ], len + _section.indices[ i + 2 ] );
			}
		}
		indices = indices.concat( indices.copy().reverse() );
		//updates geometry
		updateVertexData( vs );
		autoDeriveVertexNormals = true;
		updateIndexData( indices );
		updateUVData( uvs );
	}
	/**
	 * recomputes the vertices positions only
	 */
	public function update():Void
	{
		var vs:Vector<Float> = new Vector<Float>();
		var v:Vector3D, n:Vector3D, dir:Vector3D, angle:Float = 0;
		var sectionVectors:Vector<Vector3D>;
		for ( var i:Int = 0; i < path.length )
		{
			v = path[ i ];
			n = path[ ( i + 1 ) >= path.length ? i : ( i+1 ) ];
			if ( !v.equals( n ) )
			{
				dir = v.subtract( n );
				dir.normalize();
				angle = Math.acos( FIXED_UP_AXIS.dotProduct( dir ) ) * 180 / Math.PI;
				dir = FIXED_UP_AXIS.crossProduct(dir);
				dir.normalize();
				_section.transform.identity();
				_section.rotate( dir, angle );
			}
			_section.position = v;
			sectionVectors = _section.vectors3D;
			for( v in sectionVectors )vs.push( v.x, v.y, v.z );
		}
		updateVertexData( vs );
	}
	/*******************************************
	properties
	*******************************************/
	public var path( get_path, set_path ):Vector;
	public function get_path():Vector<Vector3D> 
	{
		return _path;
	}
	public function set_path(value:Vector<Vector3D>):Vector 
	{
		_path = value;
		compute();
		return path;
	}
	public var radius( get_radius, set_radius ):Float;
	public function get_radius():Float 
	{
		return _radius;
	}
	public function set_radius(value:Float):Float 
	{
		_radius = value;
		_section = Section.createRegularSection( radius, sides );
		compute();
		return radius;
	}
	public var sides( get_sides, set_sides ):Int;
	public function get_sides():Int 
	{
		return _sides;
	}
	public function set_sides(value:Int):Int 
	{
		_sides = Math.max( 3, value );
		_section = Section.createRegularSection( radius, sides );
		compute();
		return sides;
	}
	public var capHoles( get_capHoles, set_capHoles ):Bool;
	public function get_capHoles():Bool 
	{
		return _capHoles;
	}
	public function set_capHoles(value:Bool):Bool 
	{
		_capHoles = value;
		compute();
		return capHoles;
	}
	/*************************************************
		utils
	*************************************************/
	public var mesh( get_mesh, null ):Mesh;
	public function get_mesh():Mesh
	{
		if ( _mesh == null )
		{
			_mesh = new Mesh();
			return addToMesh( _mesh );
		}
		return _mesh;
	}
	public function addToMesh( mesh:Mesh ):Mesh
	{
		mesh.geometry.addSubGeometry( this );
		var subMesh:SubMesh = mesh.getSubMeshForSubGeometry( this );
		subMesh.material = material;
		return mesh;
	}
}