package triga.systems.verlet3d;
import nme.Vector;
using nme.Vector;
import away3d.core.base.Object3D;
import nme.display.IDrawCommand;
import nme.geom.Vector3D;
import nme.utils.clearInterval;
import nme.utils.getTimer;
import nme.utils.setInterval;
import triga.systems.verlet3d.deflectors.IDeflector;
import triga.systems.verlet3d.deflectors.PlaneDeflector;
import triga.systems.verlet3d.forces.Force;
import triga.systems.verlet3d.forces.Gravity;
import triga.systems.verlet3d.forces.IForce;
import triga.systems.verlet3d.deflectors.Deflector;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class Verlet 
{
	private var v:Vector3D;
	private var _iterations:Int = 1;
	private var _framerate:Int = 30;
	private var interval:Int;
	private var _oldTime:Float;
	private var time:Float;
	private var playing:Bool;
	private var _onUpdateClosure:Function;
	private var points:Vector<Int> = new Vector<Int>();
	private var availableSlots:Vector<Int> = new Vector<Int>();
	private var fixed:Vector<Boolean> = new Vector<Boolean>();
	private var masses:Vector<Float> = new Vector<Float>();
	private var frictions:Vector<Float> = new Vector<Float>();
	private var positions_x:Vector<Float> = new Vector<Float>();
	private var positions_y:Vector<Float> = new Vector<Float>();
	private var positions_z:Vector<Float> = new Vector<Float>();
	private var velocities_x:Vector<Float> = new Vector<Float>();
	private var velocities_y:Vector<Float> = new Vector<Float>();
	private var velocities_z:Vector<Float> = new Vector<Float>();
	private var forces_x:Vector<Float> = new Vector<Float>();
	private var forces_y:Vector<Float> = new Vector<Float>();
	private var forces_z:Vector<Float> = new Vector<Float>();
	private var sticks:Vector<Int> = new Vector<Int>();
	private var sticks_length:Vector<Float> = new Vector<Float>();
	private var chains:Vector<VerletChain> = new Vector<VerletChain>();
	private var objects:Vector<Object3D> = new Vector<Object3D>();
	private var forces:Vector<IForce> = new Vector<IForce>();
	private var deflectors:Vector<IDeflector> = new Vector<IDeflector>();
	public function new( framerate:Int = 30, onUpdate:Function = null ) 
	{ 
		this.framerate = framerate;
		_onUpdateClosure = onUpdate;
	}
	public function start():Void
	{
		playing = true;
		_oldTime = getTimer();
		clearInterval( interval );
		interval = setInterval( update, 1000 / framerate );
	}
	public function stop():Void
	{
		playing = false;
		clearInterval( interval );
	}
	public function addPoint( x:Float, y:Float, z:Float, mass:Float = 1, friction:Float = 1 ):Int
	{
		var id:Int;
		if ( availableSlots.length > 0 )
		{
			id = addPointAt( availableSlots[0], x, y, z, mass, friction );
			availableSlots.shift();
		}
		else
		{
			id = points.length;
			points.push( id );
			masses.push( Math.max( 0, mass ) );
			frictions.push( Math.max( 0, Math.min( friction, 1 ) ) );
			positions_x.push( x );
			positions_y.push( y );
			positions_z.push( z );
			velocities_x.push( 0 );
			velocities_y.push( 0 );
			velocities_z.push( 0 );
			forces_x.push( 0 );
			forces_y.push( 0 );
			forces_z.push( 0 );
			objects.push( null );
		}
		return id;
	}
	public function addPointAt( id:Int, x:Float, y:Float, z:Float, mass:Float = 1, friction:Float = 1 ):Int
	{
		points[ id ] = id;
		setMass( id, mass );
		setFriction( id, friction );
		setPoistion( id, x, y, z);
		setVelocity( id, 0,0,0 );
		setForce( id, 0, 0, 0 );
		objects[ id ] = null;
		return id;
	}
	public function removePoint( id:Int ):Void
	{
		points[ id ] = NaN;
		availableSlots.push( id );
	}
	public function getVector3D( id:Int ):Vector3D
	{
		return new Vector3D( 	positions_x[ id ],
								positions_y[ id ],
								positions_z[ id ]	);
	}
	/*****************************
	parameters
	*****************************/
	public function setMass( id:Int, mass:Float ):Void
	{
		masses[ id ] = Math.max( 0, mass );
	}
	public function setFriction( id:Int, friction:Float ):Void
	{
		friction[ id ] = Math.max( 0, Math.min( friction, 1 ) );
	}
	public function setPoistion( id:Int, x:Float, y:Float, z:Float ):Void
	{
		positions_x[ id ] = x;
		positions_y[ id ] = y;
		positions_z[ id ] = z;
	}
	public function setVelocity( id:Int, vx:Float, vy:Float, vz:Float ):Void
	{
		velocities_x[ id ] = vx;
		velocities_y[ id ] = vy;
		velocities_z[ id ] = vz;
	}
	public function getVelocity( id:Int ):Vector3D
	{
		return new Vector3D( 	velocities_x[ id ],
								velocities_y[ id ],
								velocities_z[ id ]);
	}
	public function setForce( id:Int, fx:Float, fy:Float, fz:Float ):Void
	{
		forces_x[ id ] = fx;
		forces_y[ id ] = fy;
		forces_z[ id ] = fz;
	}
	public function addStick( p0:VerletPoint3d, p1:VerletPoint3d, length:Float = -1 ):Int
	{
		sticks.push( new VerletStick3d( p0, p1, length ) );
		return sticks.length - 1;
	}
	/*****************************
	update loop
	*****************************/
	private function update():Void
	{
		//time delta
		var T:Int = getTimer();
		time = (T - _oldTime) / 1000;
		_oldTime = T;
		v = new Vector3D();
		var len:Int = points.length;
		var id:Int;
		var mass:Float;
		var friction:Float;
		var a1:Vector3D = new Vector3D(), a2:Vector3D = new Vector3D(), a3:Vector3D = new Vector3D(), a4:Vector3D = new Vector3D();
		var v1:Vector3D = new Vector3D(), v2:Vector3D = new Vector3D(), v3:Vector3D = new Vector3D(), v4:Vector3D = new Vector3D();
		for (i in 0 ... len ) 
		{
			id = points[ i ];
			//removed
			if ( Math.isNaN( id ) ) continue;
			//fixed
			mass = masses[ id ];
			if ( mass == 0 ) continue;
			friction = frictions[ id ];
			//apply forces to influence velocity
			applyForces( id );
			//dirty integrator
			velocities_x[ id ] += forces_x[ id ] * mass;
			velocities_y[ id ] += forces_y[ id ] * mass;
			velocities_z[ id ] += forces_z[ id ] * mass;
			forces_x[ id ] = forces_y[ id ] = forces_z[ id ] = 0;
			//applye deflectors to influence velocity
			applyDeflectors( id );
			positions_x[ id ] += velocities_x[ id ];
			positions_y[ id ] += velocities_y[ id ];
			positions_z[ id ] += velocities_z[ id ];
			velocities_x[ id ] *= friction;
			velocities_y[ id ] *= friction;
			velocities_z[ id ] *= friction;
			//update the mesh
			if ( objects[ id ] != null )
			{
				v.x = objects[ id ].x = positions_x[ id ];
				v.y = objects[ id ].y = positions_y[ id ];
				v.z = objects[ id ].z = positions_z[ id ];
				objects[ id ].lookAt( new Vector3D( v.x + velocities_x[ id ],
													v.y + velocities_y[ id ],
													v.z + velocities_z[ id ]	) );
			}
		}
		if ( _onUpdateClosure != null ) _onUpdateClosure();
	}
	public function onUpdate(closure:Function ):Void 
	{
		_onUpdateClosure = closure;
	}
	/***********************************
	forces and deflectors
	***********************************/
	public function addForce( force:Force ):Int
	{
		force.positions_x = positions_x;
		force.positions_y = positions_y;
		force.positions_z = positions_z;
		force.forces_x = forces_x;
		force.forces_y = forces_y;
		force.forces_z = forces_z;
		forces.push( force ); 
		return forces.length - 1;
	}
	public function removeForce( force:Force ):Int
	{
		forces.splice( forces.indexOf( force ), 1 ); 
		return forces.length - 1;
	}
	public function addDeflector( deflector:Deflector ):Int
	{
		deflector.positions_x = positions_x;
		deflector.positions_y = positions_y;
		deflector.positions_z = positions_z;
		deflector.velocities_x = velocities_x;
		deflector.velocities_y = velocities_y;
		deflector.velocities_z = velocities_z;
		deflectors.push( deflector ); 
		return deflectors.length - 1;
	}
	public function removeDeflector( deflector:Deflector ):Int
	{
		deflectors.splice( deflectors.indexOf( deflector ), 1 ); 
		return deflectors.length - 1;
	}
	private function applyForces( id:Int ):Void
	{
		for( var f:Force in forces )
		{
			f.apply( id );
		}
	}
	private function applyDeflectors( id:Int ):Void
	{
		for( var d:Deflector in deflectors )
		{
			d.apply( id );
		}
	}
	/***********************************
	3D objects associations
	***********************************/
	public function addObject3D( id:Int, object:Object3D ):Void
	{
		objects[ id ] = object;
	}
	public function getObject3D( id:Int ):Object3D
	{
		return objects[ id ];
	}
	public function removeObject3D( id:Int ):Void
	{
		objects[ id ] = null;
	}
	/***********************************
	simulation properties
	***********************************/
	public var framerate( get_framerate, set_framerate ):Int;
	public function get_framerate():Int 
	{
		return _framerate;
	}
	public function set_framerate(value:Int):Int 
	{
		var p:Bool = playing;
		if( p ) stop();
		_framerate = value;
		if ( p ) start();
		return framerate;
	}
}