package triga.utils;
import nme.Vector;
using nme.Vector;
/**
 * @author Nicolas Barradeau
 * http://en.nicoptere.net
 */
class VectorUtils 
{
	/**
	 * shrinks a Vector<Float> down to a given length
	 * @param	values original vector of values
	 * @param	length the desired length
	 * @return a vector of values matching the desired length
	 */
	static public function compress( values:Vector<Float>, length:Int ):Vector<Float>
	{
		var tmp:Vector<Float> = new Vector<Float>();
		var i:Float;
		if ( values.length == 1 )
		{
			i = length;
			while( i-- > 0 ) tmp.push( values[0] );
			return tmp;
		}
		var inputLength:Int = values.length - 2;
		var outputLength:Int = length - 2;
		tmp.push( values[ 0 ] );
		i = 0;
		var j:Int = 0;
		while (j < inputLength) 
		{
			var diff:Int = ( i + 1 ) * inputLength - ( j + 1 ) * outputLength;
			if (diff < inputLength / 2 ) 
			{
				i++;
				tmp.push( values[ j++ ] );
			}
			else j++;
		}
		tmp.push( values[ inputLength + 1 ] );
		return tmp;
	}
	/**
	 * compresses 2 vectors of numbers
	 * @param	u first set of vectors to compress
	 * @param	v second set of vectors to compress
	 * @param	length0 the desired length for vector u
	 * @param	length1 the desired length for vector v
	 * @return a Vector<Float> of doublets (x,y) of the given length interpolating the original values
	 */
	static public function compress2D( u:Vector<Float>, v:Vector<Float>, length0:Int, length1:Int ):Vector<Float>
	{
		var _u:Vector<Float> = compress( u, length0 );
		var _v:Vector<Float> = compress( v, length1 );
		var tmp:Vector<Float> = new Vector<Float>();
		for (i in 0 ... length0) 
		{
			for (j in 0 ... length1) 
			{
				tmp.push( _u[ i ], _v[ j ] );
			}
		}
		return tmp;
	}
	/**
	 * compresses 3 vectors of numbers
	 * @param	u first set of vectors to compress
	 * @param	v second set of vectors to compress
	 * @param	w second set of vectors to compress
	 * @param	length0 the desired length for vector u
	 * @param	length1 the desired length for vector v
	 * @param	length2 the desired length for vector w
	 * @return a Vector<Float> of triplets (x,y,z) of the given length interpolating the original values
	 */
	static public function compress3D( u:Vector<Float>, v:Vector<Float>, w:Vector<Float>, length0:Int, length1:Int, length2:Int ):Vector<Float>
	{
		var _u:Vector<Float> = compress( u, length0 );
		var _v:Vector<Float> = compress( v, length1 );
		var _w:Vector<Float> = compress( w, length2 );
		var tmp:Vector<Float> = new Vector<Float>();
		for (i in 0 ... length0) 
		{
			for (j in 0 ... length1) 
			{
				for (k in 0 ... length2) 
				{
					tmp.push( _u[ i ], _v[ j ], _w[ k ] );
				}
			}
		}
		return tmp;
	}
	/**
	 * expands the content of a vector.<Float> to match a given length
	 * @param	values original vector of values
	 * @param	length the desired length of the new Vector
	 * @return a Vector<Float> of the given length interpolating the original values
	 */
	static public function expand( values:Vector<Float>, length:Int ):Vector<Float>
	{
		var tmp:Vector<Float> = new Vector<Float>();
		var i:Float;
		if ( values.length == 1 )
		{
			i = length;
			while( i-- > 0 ) tmp.push( values[0] );
			return tmp;
		}
		var step:Float = 1 / ( ( values.length-1 ) / (length-1) );
		for ( i in 0... length; i+= step ) 
		{
			var val0:Float = values[ Math.floor( i / step ) ];
			var max:Int = ( Math.floor( i / step ) + 1 );
			if ( max > values.length - 1 ) max = values.length - 1;
			var val1:Float = values[ max ];
			var delta:Int = ( i + step > length ) ? ( length - i ) : step;
			for (var j:Float = 0; j < delta) 
			{ 
				tmp.push( lerp( j / delta, val0, val1 ) );
			}
		}
		while ( tmp.length < length ) tmp.push( values[ values.length - 1 ] );
		return tmp;
	}
	/**
	 * expands 2 vectors of numbers and combines them together
	 * @param	u first set of vectors to expand
	 * @param	v second set of vectors to expand
	 * @param	length0 the desired length for vector u
	 * @param	length1 the desired length for vector v
	 * @return a Vector<Float> of doublets (x,y) of the given length interpolating the original values
	 */
	static public function expand2D( u:Vector<Float>, v:Vector<Float>, length0:Int, length1:Int ):Vector<Float>
	{
		var _u:Vector<Float> = expand( u, length0 );
		var _v:Vector<Float> = expand( v, length1 );
		var tmp:Vector<Float> = new Vector<Float>();
		for (i in 0 ... length0) 
		{
			for (j in 0 ... length1) 
			{
				tmp.push( _u[ i ], _v[ j ] );
			}
		}
		return tmp;
	}
	/**
	 * expands 3 vectors of numbers
	 * @param	u first set of vectors to expand
	 * @param	v second set of vectors to expand
	 * @param	w second set of vectors to expand
	 * @param	length0 the desired length for vector u
	 * @param	length1 the desired length for vector v
	 * @param	length2 the desired length for vector w
	 * @return a Vector<Float> of triplets (x,y,z) of the given length interpolating the original values
	 */
	static public function expand3D( u:Vector<Float>, v:Vector<Float>, w:Vector<Float>, length0:Int, length1:Int, length2:Int ):Vector<Float>
	{
		var _u:Vector<Float> = expand( u, length0 );
		var _v:Vector<Float> = expand( v, length1 );
		var _w:Vector<Float> = expand( w, length2 );
		var tmp:Vector<Float> = new Vector<Float>();
		for (i in 0 ... length0) 
		{
			for (j in 0 ... length1) 
			{
				for (k in 0 ... length2) 
				{
					tmp.push( _u[ i ], _v[ j ], _w[ k ] );
				}
			}
		}
		return tmp;
	}
	static private function lerp( t:Float, min:Float, max:Float):Float
	{
		return min + ( max - min ) * t;
	}
}